package dev.fitko.fitconnect.client.strategies;

import dev.fitko.fitconnect.api.domain.model.submission.CreateSubmission;
import dev.fitko.fitconnect.api.domain.model.submission.SubmissionForPickup;
import dev.fitko.fitconnect.api.domain.model.submission.SubmitSubmission;
import dev.fitko.fitconnect.api.exceptions.RestApiException;
import dev.fitko.fitconnect.api.exceptions.SubmissionNotCreatedException;
import dev.fitko.fitconnect.api.services.Sender;
import dev.fitko.fitconnect.client.model.AttachmentPayload;
import dev.fitko.fitconnect.client.model.SubmissionPayload;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.Optional;
import java.util.UUID;

import static dev.fitko.fitconnect.client.util.SubmissionUtil.buildSubmissionToAnnounce;
import static dev.fitko.fitconnect.client.util.SubmissionUtil.buildSubmitSubmission;

public class SendEncryptedSubmissionStrategy implements SubmitStrategy {

    private static final Logger LOGGER = LoggerFactory.getLogger(SendEncryptedSubmissionStrategy.class);

    private final Sender sender;

    public SendEncryptedSubmissionStrategy(final Sender sender) {
        this.sender = sender;
    }

    @Override
    public Optional<SubmitSubmission> send(final SubmissionPayload submissionPayload) {
        try {
            final UUID submissionId = announceNewSubmission(submissionPayload);
            final SubmitSubmission submission = buildSubmitSubmission(submissionId, submissionPayload);
            uploadAttachments(submissionPayload.getAttachments(), submissionId);
            sender.sendSubmission(submission);
            LOGGER.info("SUCCESSFULLY HANDED IN SUBMISSION !");
            return Optional.of(submission);

        } catch (final RestApiException e) {
            LOGGER.error("Sending submission failed", e);
        } catch (final SubmissionNotCreatedException e) {
            LOGGER.error("Failed to announce new submission", e);
        }
        return Optional.empty();
    }

    private UUID announceNewSubmission(final SubmissionPayload submissionPayload) {
        final CreateSubmission submissionToAnnounce = buildSubmissionToAnnounce(submissionPayload);
        final SubmissionForPickup announcedSubmission = sender.createSubmission(submissionToAnnounce);
        return announcedSubmission.getSubmissionId();
    }
    private void uploadAttachments(final List<AttachmentPayload> attachmentPayloads, final UUID submissionId) {
        if (attachmentPayloads.isEmpty()) {
            LOGGER.info("No attachments to upload");
        } else {
            LOGGER.info("Uploading {} attachment(s)", attachmentPayloads.size());
        }
        for (final AttachmentPayload payload : attachmentPayloads) {
            sender.uploadAttachment(submissionId, payload.getAttachmentId(), payload.getEncryptedData());
        }
    }
}
